package grpctool

import (
	"net"
	"strconv"

	"gitlab.com/gitlab-org/cluster-integration/gitlab-agent/v17/internal/tool/nettool"
	"google.golang.org/grpc/resolver"
	"google.golang.org/grpc/resolver/manual"
)

const (
	MultiURLResolverScheme = "multiurl"
	MultiURLResolverTarget = MultiURLResolverScheme + ":///"
)

func NewMultiURLResolver(mu nettool.MultiURL) *manual.Resolver {
	muAddrs := mu.Addresses()
	addrs := make([]resolver.Address, 0, len(muAddrs))
	portStr := strconv.FormatUint(uint64(mu.Port()), 10)
	for _, uAddr := range muAddrs {
		uAddrStr := uAddr.String()
		// There is no host in MultiURLResolverTarget so for TLS we should set at least the IP as the server name.
		// This can be overridden via the tls.Config#ServerName.
		addrs = append(addrs, resolver.Address{
			Addr:       net.JoinHostPort(uAddrStr, portStr),
			ServerName: uAddrStr,
		})
	}
	b := manual.NewBuilderWithScheme(MultiURLResolverScheme)
	b.InitialState(resolver.State{
		Endpoints: []resolver.Endpoint{
			{
				Addresses: addrs,
			},
		},
	})
	return b
}
