package nettool

import (
	"net/netip"
	"testing"

	"github.com/stretchr/testify/assert"
	"github.com/stretchr/testify/require"
)

func TestParseMultiURL(t *testing.T) {
	ip1234 := netip.MustParseAddr("1.2.3.4")
	ipv6 := netip.MustParseAddr("123::123")
	tests := []struct {
		input       string
		expected    MultiURL
		expectedErr string
	}{
		{
			input:       "",
			expectedErr: "scheme not found in: ",
		},
		{
			input:       "grpc:",
			expectedErr: "port not found in: grpc:",
		},
		{
			input:       "grpc:|",
			expectedErr: `port in grpc:|: strconv.ParseUint: parsing "": invalid syntax`,
		},
		{
			input:       "grpc:|123",
			expectedErr: `IP parsing in grpc:|123: ParseAddr(""): unable to parse IP`,
		},
		{
			input:       "grpc:1.2.3.4|99999",
			expectedErr: `port in grpc:1.2.3.4|99999: strconv.ParseUint: parsing "99999": value out of range`,
		},
		{
			input: "grpc:1.2.3.4|1234",
			expected: MultiURL{
				scheme:    "grpc",
				addresses: []netip.Addr{ip1234},
				port:      1234,
			},
		},
		{
			input: "grpc:1.2.3.4,123::123|1234",
			expected: MultiURL{
				scheme:    "grpc",
				addresses: []netip.Addr{ip1234, ipv6},
				port:      1234,
			},
		},
	}
	for _, tc := range tests {
		t.Run(tc.input, func(t *testing.T) {
			ma, err := ParseMultiURL(tc.input)
			if tc.expectedErr == "" {
				require.NoError(t, err)
				assert.True(t, IsEqualMultiURL(tc.expected, ma))
				assert.Equal(t, tc.input, ma.String())
			} else {
				assert.EqualError(t, err, tc.expectedErr)
			}
		})
	}
}
